/****************************************************************/
/*																*/
/*	White Glove Tracking 										*/
/*	www.whiteglovetracking.com									*/
/*	06.26.07													*/
/*	evan@ni9e.com												*/
/*																*/
/****************************************************************/

#include "testApp.h"
#include "stdio.h"

//--------------------------------------------------------------
testApp::testApp(){
	
}

//--------------------------------------------------------------
void testApp::setup(){	 
	ofBackground(255,255,255);	
	
	GC = new grayScaleConverter();
	threshold = 150;
	fade_counter = 0;
	
	//image capture stuff
	snapCounter = 0;
	bSnapshot = false;
	phase = 0;
	memset(snapString, 0, 255);		// clear the string by setting all chars to 0
	
	mov_w = 640;
	mov_h = 480;
	
	//texGray.allocate(mov_w,mov_h,GL_LUMINANCE);
	texGlove.allocate(mov_w,mov_h,GL_LUMINANCE);
	grayPixels	= new unsigned char [mov_w * mov_h];
	glovePixels	= new unsigned char [mov_w * mov_h];
	
	//set all pixels to black
	for(int i=0; i<640*480; i++){
		glovePixels[i] = 0;
	}
	
	//load in data from text file to IO object
	IO = new dataIO();
	IO->load_data("wgt_data_v1.txt");
	printf("There are %d frames total\n", IO->frame);
	
	//load movie
	billyjean.loadMovie("billyjean_640_480_65mb.mov");
	movDuration = billyjean.getDuration();
	
}

//--------------------------------------------------------------
void testApp::update(){
	
	//update movie, time, frame
	billyjean.idleMovie();
	movTime = movDuration * billyjean.getPosition();
	movFrame = int(29.97 * movTime);
	
	//convert to greyscale
	unsigned char * bj_pixels = billyjean.getPixels();
	GC->convert(bj_pixels,grayPixels,mov_w,mov_h);

	//write glove pixels into image
	int xstart 	= 	IO->x_pts[movFrame];
	int xend	=	IO->x_pts[movFrame] + IO->w_pts[movFrame];
	int ystart	=	IO->y_pts[movFrame];
	int yend	=	IO->y_pts[movFrame] + IO->h_pts[movFrame];
	
	for (int x = xstart; x < xend; x++) {
	    for (int y = ystart; y < yend; y++ ) {
	      int loc = x + y*mov_w;
	      if(grayPixels[loc] > threshold){
	      	glovePixels[loc] = grayPixels[loc];
	      	
	      	//brighten up image a bit
	      	int brighten = 25;
	      	if(glovePixels[loc] + brighten > 255){
	      		glovePixels[loc] = 255;
	      	}else{
	      		glovePixels[loc] += brighten;
	      	} 
	      	
	      }
	    }
  	}
  	
   	//fade image
  	fade_counter++;
  	if(fade_counter == 45){
		for(int k=0; k<640*480; k++){
			if(glovePixels[k] > 0){
				glovePixels[k] -= 1;
			}
		}
		fade_counter = 0;
	}

	//load pix into textures
	texGlove.loadData(glovePixels, mov_w,mov_h, GL_LUMINANCE);
}

//--------------------------------------------------------------
void testApp::draw(){
	ofSetColor(0xFFFFFF);

	texGlove.draw(0,0,ofGetWidth(), ofGetHeight());
	
	//take screen grab to .png
	if (bSnapshot == true){
		// grab a rectangle at 200,200, width and height of 300,180
		img.grabScreen(0,0,640,480);
		char fileName[255];
		sprintf(fileName, "snapshot_%0.3i.png", snapCounter);
		img.saveImage(fileName);
		sprintf(snapString, "saved %s", fileName);
		snapCounter++;
		bSnapshot = false;
	}
}

//--------------------------------------------------------------
void testApp::keyPressed  (int key){ 
	if (key == 'x'){
		bSnapshot = true;
	}
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
}

//--------------------------------------------------------------
void testApp::mouseReleased(){
}
